
import pypipet.core.shop_conn.wc as wc
import pypipet.core.transform.wc_to_model as wc_to_model
import pypipet.core.transform.model_to_wc as model_to_wc
from .shop_connector import ShopConnector
import logging
logger = logging.getLogger('__default__')

# def _clean_up_data(data):
#     keys = set(list(data.keys()))
#     if 'created_at' in keys:
#         del data['created_at']
#     if 'updated_at' in keys:
#         del data['updated_at']


# def validate_product_info(product_info):
#     if product_info.get('images') is None:
#         logger.debug('missing images info')
#         return False

#     if product_info.get('price') is None:
#         logger.debug('missing price info')

#     if product_info.get('sku') is None:
#         logger.debug('missing sku info')

#     return True

class WCShopConnector(ShopConnector):
    def __init__(self, shop_name, shop_type, **kwargs):
        super().__init__(shop_name, shop_type, **kwargs)
        self.batch_size = kwargs.get('batch_size', 10)

    

    def add_product_to_shop(self, product_info, **kwargs):
        attr_list = kwargs.get('attr_list', ['brand', 'upc', 'size', 'color'])
        if kwargs.get('variation_attrs') is None or product_info.get('variations') is None:
            if model_to_wc.parse_to_wp_product(self.shop_api, product_info, attr_list, 
                                        status=kwargs.get('publish_type', 'publish')):
                return wc.add_product(self.shop_api, product_info)
        else:
            product = model_to_wc.parse_to_wp_product_variable(self.shop_api, product_info, 
                                            attr_list,kwargs['variation_attrs'],
                                            parent_id=kwargs.get('parent_id'))
            
            if product:
                return wc.add_product_variations(self.shop_api, product, parent_id=kwargs.get('parent_id'))


    def add_product_to_shop_bulk(self, products:list,  **kwargs):
        """for woocommerce, bulk add only for non-variable product"""
        attr_list = kwargs.get('attr_list', ['brand', 'upc', 'size', 'color'])
        batch = []
        for product_info in products:
            if model_to_wc.parse_to_wp_product(self.shop_api, product_info, attr_list):
                batch.append(product_info)
        #batch add 
        res = wc.update_product_batch(self.shop_api, 'create', batch)



    def get_destination_product_id(self, sku:str):
        """
        shop: dict with keys (front_shop_id, shop_api, shop_type)
        """
        product = wc.get_product_by_sku(self.shop_api, sku)
        if product:
            return product['id']

        return None

    def update_shop_product(self, data:dict, product_id:str, **kwargs ):
        attr_list = kwargs.get('attr_list', ['brand', 'upc', 'size', 'color'])
        if kwargs.get('formated') is None or kwargs['formated'] == False:
            if not model_to_wc.parse_to_wp_product(self.shop_api, data, attr_list, update_only=True):
                logger.debug('wp parsing failed')
                return 
        # print(data)
        if kwargs.get('parent_id') is not None:
            # print(kwargs['parent_id'], data)
            return wc.update_variation(self.shop_api, kwargs['parent_id'], product_id, data)
        return wc.update_product(self.shop_api, product_id, data)

    def update_shop_product_batch(self, data:list, **kwargs):
        attr_list = kwargs.get('attr_list', ['brand', 'upc', 'size', 'color'])
        batch = []
        for i, d in enumerate(data):
            # print(d)
            if kwargs.get('formated') is None or kwargs['formated'] == False:
                if model_to_wc.parse_to_wp_product(self.shop_api, d, attr_list, update_only=True):
                    if d.get('parent_id') is not None:
                        # print(d)
                        wc.update_variation(self.shop_api, d['parent_id'] , d['id'], d)
                    else:
                        batch.append(d)
            else:
                batch.append(d)
        
        res = wc.update_product_batch(self.shop_api, 'update', batch)
        return res

    def sync_shop_orders(self,  **kwargs):
        res = []
        wp_orders = wc.get_new_orders(
            self.shop_api,
            start_from_order=kwargs.get('latest_order_id', '0'), 
            page_start=kwargs.get('page_start', 1), 
            per_page=kwargs.get('per_page', 20))
    
        for order in wp_orders:
            print('prcessing order', order['id'])
            # todo: get tax based on geo
            data = wc_to_model.wp_parse_order(order, 
                            kwargs.get('field_mapping', self.field_mapping), 
                            shipping_tax_id=kwargs.get('shipping_tax_id',2), 
                            item_tax_id=kwargs.get('item_tax_id', 1)) 
            res.append(data)

        return res


    def update_order_status_at_shop(self, destination_order_id, 
                                   status,**kwargs):
        res = wc.update_order(self.shop_api,
                            destination_order_id,
                            data={'status': status})
        return res

    def update_order_at_shop(self, destination_order_id, 
                             data, **kwargs):
        res = wc.update_order(self.shop_api,
                            destination_order_id,
                            data=data)
        return res

    def update_order_item_at_shop(self, destination_order_id, 
                             items, **kwargs):
        order = wc.get_order_by_id(self.shop_api,
                                   destination_order_id)
        item_list = model_to_wc.transform_order_item(order, items)
        # print(item_list)
        res = wc.update_order(self.shop_api,
                            destination_order_id,
                            data={'line_items': item_list})
        return res

    def get_order_at_shop(self, **kwargs):
        # print(kwargs.get('destination_order_id'))
        res = wc.get_order_by_id(self.shop_api,
                            kwargs.get('destination_order_id'))
        # todo: get tax based on geo
        res = wc_to_model.wp_parse_order(res, kwargs.get('field_mapping', self.field_mapping), 
                        shipping_tax_id=kwargs.get('shipping_tax_id',2), 
                        item_tax_id=kwargs.get('item_tax_id', 1)) 
        return res

            

    def get_products_at_shop(self, **kwargs):
        """wc page start from 1"""
        data = wc.list_products(self.shop_api, 
                            params={
                                'page': kwargs.get('page', 1), 
                                'per_page': kwargs.get('batch_size', 10),
                                'status': kwargs.get('status', 'publish')
                            }) 
        for i, d in enumerate(data):
            data[i] = self._parse_product(d, 
                                    kwargs.get('field_mapping', self.field_mapping))
        return data
                
                            
    def get_product_at_shop(self, product_id, **kwargs):
        if kwargs.get('parent_id') is not None:
            product_id = kwargs['parent_id']
        res = wc.get_product_by_id(self.shop_api, product_id) 
        return self._parse_product(res, 
                                    kwargs.get('field_mapping', self.field_mapping))
            
                

    def get_category_taxonomy(self, **kwargs):
        page = 1
        cats = []
        while True:
            batch_cats = wc.list_categories(self.shop_api, 
                                    page, 
                                    params=kwargs) 
            if batch_cats is None:
                return None
            if len(batch_cats) == 0:
                break
            cats += batch_cats 
            page += 1
        wc_to_model.wp_category_to_taxonomy(cats)
        return cats
    
    def update_customer_at_shop(self, destination_order_id, 
                             data, **kwargs):
        model_to_wc.transform_customer(data)
        update = {}
        if kwargs.get('is_billing'):
            update['billing'] = data
        elif kwargs.get('is_shipping'):
            update['shipping'] = data
        else:
            logger.debug('need to specify biling or shipping')
            return 
        return wc.update_order(self.shop_api, destination_order_id, update) 

    def add_tracking(self, destination_order_id, 
                            message, **kwargs):
        assert message.get('tracking_id')
        assert message.get('provider')
        tracking = f"tracking code {message['tracking_id']} service by \
                        {message['provider']}"
        res = wc.send_wc_message(self.shop_api,
                            destination_order_id,
                            tracking)
        return res

    def send_message(self, destination_order_id, 
                            message, **kwargs):
        res = wc.send_wc_message(self.shop_api,
                            destination_order_id,
                            message)
        return res

    def _parse_product(self, product, field_mapping):
        parsed = wc_to_model.parse_product(product, field_mapping)
        if parsed == 0:
            return 
        elif parsed == 1:
            variations = []
            for pid in product['variations']:
                vari = wc.get_product_by_id(self.shop_api, pid) 
                if vari: variations.append(vari)
            product['variations'] = variations 
            
            parsed = wc_to_model.parse_product_with_variations(product, field_mapping)
            
        return parsed



import pandas as pd
import yaml
import shutil


def write_csv_file(data:list, filename):
    df = pd.DataFrame(data)
    return df.to_csv(filename,
                index=False)


def _model_to_dict(obj):
    try:
        return obj.get_all_attrs()
    except Exception as e:
        print(e)

def write_yaml(data, filename):
    with open(filename, 'w') as outfile:
        yaml.dump(data, outfile)

def copy_file(src, dst):
    shutil.copyfile(src, dst)


from requests import request
from json import dumps as jsonencode
from time import time
from pypipet.plugins.woocommerce.oauth import OAuth
from requests.auth import HTTPBasicAuth
from urllib.parse import urlencode
from oauthlib.oauth2 import BackendApplicationClient
from requests_oauthlib import OAuth2Session
from oauthlib.common import urlencode
import base64, time

def encode_base64(text):
    return  base64.b64encode(bytes(text, 'utf-8'))

class API(object):
    """ API Class """

    def __init__(self, url, client_id, secret, **kwargs):
        self.error = None
        if url.endswith("/") is False:
            self.url = f"{url}/"
        else:
            self.url = url
        self._AUTH_URL =  kwargs.get("auth_url", "v1/oauth2/token")
        self.client_id = client_id
        self.secret = secret
        self.version = kwargs.get("version", "v1")
        self.is_ssl = self.__is_ssl()
        self.timeout = kwargs.get("timeout", 800)
        self.verify_ssl = kwargs.get("verify_ssl", True)
        self.__get_oauth_token(self._AUTH_URL)
        

    def __is_ssl(self):
        """ Check if url use HTTPS """
        return self.url.startswith("https")

    def __get_url(self, endpoint):
        """ Get URL for requests """
        return f"{self.url}{self.version}/{endpoint}"

    def __get_oauth_token(self, endpoint,  **kwargs):
        auth =HTTPBasicAuth(self.client_id, self.secret)
        client = BackendApplicationClient(client_id=self.client_id)
        oauth = OAuth2Session(client=client)
        url = f"{self.url}{endpoint}"
        try:
            token = oauth.fetch_token(token_url=url, 
                                    client_id=encode_base64(self.client_id),
                                    client_secret=encode_base64(self.secret), 
                                    auth=auth)
            self.token = token['access_token']
            self.expire_at = token['expires_at']
        except:
            self.error = 'oauth failed'
        

    def __request(self, method, endpoint, data, params=None, **kwargs):
        """ Do requests """

        #check if token expired
        if time.time() > self.expire_at:
            self.__get_oauth_token(self._AUTH_URL)

        if params is None:
            params = {}
        url = self.__get_url(endpoint)
        auth = None
        headers = {
            "Authorization": f"Bearer {self.token}",
            "accept": "application/json"
        }


        if data is not None:
            data = jsonencode(data, ensure_ascii=False).encode('utf-8')
            headers["content-type"] = "application/json;charset=utf-8"
        return request(
            method=method,
            url=url,
            verify=self.verify_ssl,
            auth=auth,
            params=params,
            data=data,
            timeout=self.timeout,
            headers=headers,
            **kwargs
        )

    def get(self, endpoint, **kwargs):
        """ Get requests """
        return self.__request("GET", endpoint, None, **kwargs)

    def post(self, endpoint, data, **kwargs):
        """ POST requests """
        return self.__request("POST", endpoint, data, **kwargs)

    def put(self, endpoint, data, **kwargs):
        """ PUT requests """
        return self.__request("PUT", endpoint, data, **kwargs)

    def delete(self, endpoint, **kwargs):
        """ DELETE requests """
        return self.__request("DELETE", endpoint, None, **kwargs)

    def options(self, endpoint, **kwargs):
        """ OPTIONS requests """
        return self.__request("OPTIONS", endpoint, None, **kwargs)



import requests



def request_handler(func):
    def wrapper(*args, **kwargs):
        try:
            res = func(*args, **kwargs)
            if res.status_code in [200, 201]:
                return res.json()
            logger.debug(res.json())
            return None
        except  KeyError as e:
            logger.debug('requst error')
            logger.debug(e)
    return wrapper


@request_handler
def get_status(tracking_id):
    domain = "https://www.canadapost-postescanada.ca"
    return requests.get(f"{domain}/track-reperage/rs/track/json/package/{tracking_id}/detail")


#!/usr/bin/python
#
# Copyright 2016 Google Inc. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Common utils for the Content API for Shopping samples."""

from __future__ import print_function
import argparse
import json
import logging
import os
import random
import sys
import time

import google_auth_httplib2
from googleapiclient import discovery
from googleapiclient import errors
from googleapiclient import http
from googleapiclient import model
from pypipet.plugins.gg_merchant.shopping.content import _constants
from pypipet.plugins.gg_merchant.shopping.content import auth

import six.moves.urllib.parse


# Authenticate and return the Content API service along with any command-line
# flags/arguments.
def init(argv, doc, parents=None, sandbox=False):
  """A common initialization routine for the Content API samples.

  Args:
    argv: list of string, the command-line parameters of the application.
    doc: string, description of the application. Usually set to __doc__.
    parents: list of argparse.ArgumentParser, additional command-line flags.
    sandbox: boolean, whether to use the sandbox API endpoint or not.

  Returns:
    A tuple of (service, config, flags), where service is the service object,
    config is the configuration JSON in Python form, and flags
    are the parsed command-line flags.
  """
  service = None
  sandbox_service = None
  flags = None
  parent_parsers = []
  if parents is not None:
    parent_parsers.extend(parents)

  parser = argparse.ArgumentParser(
      description=doc,
      formatter_class=argparse.RawDescriptionHelpFormatter,
      parents=parent_parsers)
  parser.add_argument(
      '--config_path',
      metavar='PATH',
      default=os.path.expanduser('~/shopping-samples'),
      help='configuration directory for the Shopping samples')
  parser.add_argument(
      '--noconfig',
      action='store_true',
      help='run samples with no configuration directory')
  parser.add_argument(
      '--log_file',
      metavar='FILE',
      help='filename for logging API requests and responses'
  )
  flags = parser.parse_args(argv[1:])

  if flags.log_file:
    logging.basicConfig(filename=flags.log_file, level=logging.INFO)
    model.dump_request_response = True

  config = {}
  if not flags.noconfig:
    if not os.path.isdir(flags.config_path):
      print(
          'Configuration directory "%s" does not exist.' % flags.config_path,
          file=sys.stderr)
      sys.exit(1)

    content_path = os.path.join(flags.config_path, 'content')
    if not os.path.isdir(content_path):
      print(
          'Content API configuration directory "%s" does not exist.' %
          content_path,
          file=sys.stderr)
      sys.exit(1)

    config_file = os.path.join(content_path, 'merchant-info.json')
    if not os.path.isfile(config_file):
      print('Configuration file %s does not exist.' % config_file)
      print('Falling back to configuration based on authenticated user.')
    else:
      config = json.load(open(config_file, 'r'))
    config['path'] = content_path

  credentials = auth.authorize(config)
  auth_http = google_auth_httplib2.AuthorizedHttp(
      credentials, http=http.set_user_agent(
          http.build_http(), _constants.APPLICATION_NAME))
  if _constants.ENDPOINT_ENV_VAR in os.environ:
    # Strip off everything after the host/port in the URL.
    root_url = six.moves.urllib.parse.urljoin(
        os.environ[_constants.ENDPOINT_ENV_VAR],
        '/')
    print('Using non-standard root for API discovery: %s' % root_url)
    discovery_url = root_url + '/discovery/v1/apis/{api}/{apiVersion}/rest'
    service = discovery.build(
        _constants.SERVICE_NAME,
        _constants.SERVICE_VERSION,
        discoveryServiceUrl=discovery_url,
        http=auth_http)
    if sandbox:
      sandbox_service = discovery.build(
          _constants.SERVICE_NAME,
          _constants.SANDBOX_SERVICE_VERSION,
          discoveryServiceUrl=discovery_url,
          http=auth_http)
  else:
    service = discovery.build(
        _constants.SERVICE_NAME, _constants.SERVICE_VERSION, http=auth_http)
    if sandbox:
      sandbox_service = discovery.build(
          _constants.SERVICE_NAME,
          _constants.SANDBOX_SERVICE_VERSION,
          http=auth_http)

  # Now that we have a service object, fill in anything missing from the
  # configuration using API calls.
  retrieve_remaining_config_from_api(service, config)

  return (sandbox_service if sandbox else service, config, flags)


unique_id_increment = 0


def get_unique_id():
  """Generates a unique ID.

  The ID is based on the current UNIX timestamp and a runtime increment.

  Returns:
    A unique string.
  """
  global unique_id_increment
  if unique_id_increment is None:
    unique_id_increment = 0
  unique_id_increment += 1
  return '%d%d' % (int(time.time()), unique_id_increment)


def retrieve_remaining_config_from_api(service, config):
  """Retrieves any missing configuration information using API calls.

  This function can fill in the following configuration fields:
  * merchantId

  It will also remove or overwrite existing values for the following fields:
  * isMCA
  * websiteUrl

  Args:
    service: Content API service object
    config: dictionary, Python representation of config JSON.
  """
  authinfo = service.accounts().authinfo().execute()
  account_ids = authinfo.get('accountIdentifiers')
  if not account_ids:
    print('The currently authenticated user does not have access to '
          'any Merchant Center accounts.')
    sys.exit(1)
  if 'merchantId' not in config:
    first_account = account_ids[0]
    config['merchantId'] = int(first_account.get('merchantId', 0))
    if not config['merchantId']:
      config['merchantId'] = int(first_account['aggregatorId'])
    print('Using Merchant Center %d for running samples.' %
          config['merchantId'])
  merchant_id = config['merchantId']
  config['isMCA'] = False
  # The requested Merchant Center can only be an MCA if we are a
  # user of it (and thus have access) and it is listed as an
  # aggregator in authinfo.
  for account_id in authinfo['accountIdentifiers']:
    if ('aggregatorId' in account_id and
        int(account_id['aggregatorId']) == merchant_id):
      config['isMCA'] = True
      break
    if ('merchantId' in account_id and
        int(account_id['merchantId']) == merchant_id):
      break
  if config['isMCA']:
    print('Merchant Center %d is an MCA.' % config['merchantId'])
  else:
    print('Merchant Center %d is not an MCA.' % config['merchantId'])
  account = service.accounts().get(
      merchantId=merchant_id, accountId=merchant_id).execute()
  config['websiteUrl'] = account.get('websiteUrl')
  if not config['websiteUrl']:
    print('No website for Merchant Center %d.' % config['merchantId'])
  else:
    print('Website for Merchant Center %d: %s' % (config['merchantId'],
                                                  config['websiteUrl']))


def is_mca(config):
  """Returns whether or not the configured account is an MCA."""
  return config.get('isMCA', False)


def check_mca(config, should_be_mca, msg=None):
  """Checks that the configured account is an MCA or not based on the argument.

  If not, it exits the program early.

  Args:
    config: dictionary, Python representation of config JSON.
    should_be_mca: boolean, whether or not we expect an MCA.
    msg: string, message to use instead of standard error message if desired.
  """
  if should_be_mca != is_mca(config):
    if msg is not None:
      print(msg)
    else:
      print('For this sample, you must%s use a multi-client account.' %
            '' if should_be_mca else ' not')
    sys.exit(1)


def retry_request(req, slot_time=5.0, max_time=60.0):
  """Retries the provided request for HTTP errors.

  Normally, we could just use the optional num_retries keyword for the
  execute() methods. However, the only 4xx errors they retry are
  429s (always) and 403s (sometimes). Unfortunately, the Content API
  sometimes returns other 4xx messages: for example, it will return 401
  if you try to retrieve a new sub-account after creating it
  before it is fully available. Here, we just retry as long as we get
  an HTTP error.

  Args:
    req: HTTP request to retry
    slot_time: float, slot time (in seconds) for retries
    max_time: float, max time (in seconds) to retry

  Returns:
    The same result as the original request, if successful.
  """
  waited_time = 0.0
  retry_num = 0
  while True:
    try:
      return req.execute()
    except errors.HttpError as e:
      if waited_time >= max_time:
        raise e
      else:
        sleep_time = random.randint(0, 2 ** retry_num - 1) * slot_time
        # Cap the sleep time to avoid overrunning the max time by too long.
        if waited_time + sleep_time > max_time:
          sleep_time = max_time - waited_time
        print('Request failed, trying again after %.2f seconds.' % sleep_time)
        time.sleep(sleep_time)
        waited_time += sleep_time
        retry_num += 1
        continue
